@@ -10,6 +10,10 @@ import logging
 import os
 import shutil
 import socket
+try:
+    import ssl
+except ImportError:
+    ssl = None
 import subprocess
 import sys
 import tempfile
@@ -17,17 +21,24 @@ try:
     import threading
 except ImportError:
     import dummy_threading as threading
+import time
 
 from compat import unittest, Request
-from support import HTTPSServerThread
+from support import DistlibTestCase, in_github_workflow
+if ssl:
+    from support import HTTPSServerThread
 
 from distlib import DistlibException
 from distlib.compat import urlopen, HTTPError, URLError
 from distlib.index import PackageIndex
 from distlib.metadata import Metadata, MetadataMissingError, METADATA_FILENAME
-from distlib.util import zip_dir, HTTPSHandler
+from distlib.util import zip_dir
+
+if ssl:
+    from distlib.util import HTTPSHandler
 
 logger = logging.getLogger(__name__)
+logger.addHandler(logging.NullHandler())
 
 HERE = os.path.abspath(os.path.dirname(__file__))
 
@@ -36,9 +47,13 @@ if 'HOME' in os.environ:
 else:
     PYPIRC = None
 
-class PackageIndexTestCase(unittest.TestCase):
+TEST_SERVER_PORT = os.environ.get('TEST_PYPISERVER_PORT', '8086')
+
+IN_GITHUB_WORKFLOW = in_github_workflow()
+
+class PackageIndexTestCase(DistlibTestCase):
     run_test_server = True
-    test_server_url = 'http://localhost:8080/'
+    test_server_url = 'http://localhost:%s/' % TEST_SERVER_PORT
 
     @classmethod
     def setUpClass(cls):
@@ -56,25 +71,56 @@ class PackageIndexTestCase(unittest.TestCase):
             pkgdir = os.path.join(HERE, 'packages')
             if not os.path.isdir(pkgdir):   # pragma: no cover
                 os.mkdir(pkgdir)
-            cls.sink = sink = open(os.devnull, 'w')
+            fd, cls.sinkfile = tempfile.mkstemp(suffix='.log', prefix='distlib-pypi-')
+            os.close(fd)
+            cls.sink = sink = open(cls.sinkfile, 'w')
             cmd = [sys.executable, 'pypi-server-standalone.py',
+                   '--interface', '127.0.0.1', '--port', TEST_SERVER_PORT,
                    '-P', 'passwords', 'packages']
             cls.server = subprocess.Popen(cmd, stdout=sink, stderr=sink,
                                           cwd=HERE)
             # wait for the server to start up
             response = None
+            tries = 20
+            timeout = 0.5
+            count = 0
             while response is None:
                 try:
+                    count += 1
+                    time.sleep(timeout)
                     response = urlopen(cls.test_server_url)
+                    if response.getcode() != 200:
+                        response = None
+                        break
+                    # In case some other server is listening on the same port ...
+                    # need to check the actual response for pypiserver-specific content
+                    data = response.read()
+                    if b'Welcome to pypiserver!' not in data:
+                        response = None
+                        break
                 except URLError:
-                    pass
+                    if count < tries:
+                        pass
+                    else:
+                        break
+            if response is None or cls.server.poll() is not None:
+                logger.warning('PyPI test server could not be run')
+                cls.server = None
+                cls.sink.close()
+                os.remove(cls.sinkfile)
 
     @classmethod
     def tearDownClass(cls):
         if cls.run_test_server:
             if cls.server and cls.server.returncode is None:
                 cls.server.kill()
+                cls.server.wait()
                 cls.sink.close()
+                try:
+                    os.remove(cls.sinkfile)
+                except:
+                    logger.warning('Unable to remove test file %s',
+                                   cls.sinkfile)
 
     def setUp(self):
         if not self.run_test_server:
@@ -128,7 +174,8 @@ class PackageIndexTestCase(unittest.TestCase):
             with open(zip_name, 'wb') as f:
                 f.write(zip_data)
 
-    def test_register(self):
+    @unittest.skip('The PyPI API changed, so this test is temporarily skipped')
+    def test_register(self):  # pragma: no cover
         "Test registration"
         self.check_pypi_server_available()
         self.check_testdist_available()
@@ -163,6 +210,7 @@ class PackageIndexTestCase(unittest.TestCase):
             if e.getcode() != 404:
                 raise
 
+    @unittest.skip('The PyPI API changed, so this test is temporarily skipped')
     def test_upload(self):
         "Test upload"
         self.check_pypi_server_available()
@@ -215,7 +263,8 @@ class PackageIndexTestCase(unittest.TestCase):
             expected = b'This is dummy documentation'
             self.assertIn(expected, data)
 
-    def test_verify_signature(self):
+    @unittest.skipIf(IN_GITHUB_WORKFLOW, 'This test is end-of-line dependent')
+    def test_verify_signature(self):  # pragma: no cover
         if not self.index.gpg:      # pragma: no cover
             raise unittest.SkipTest('gpg not available')
         sig_file = os.path.join(HERE, 'good.bin.asc')
@@ -248,58 +297,66 @@ class PackageIndexTestCase(unittest.TestCase):
         finally:
             os.remove(PYPIRC)
 
-    def make_https_server(self, certfile):
-        server = HTTPSServerThread(certfile)
-        flag = threading.Event()
-        server.start(flag)
-        flag.wait()
-        def cleanup():
-            server.stop()
-            server.join()
-        self.addCleanup(cleanup)
-        return server
+    if ssl:
+        def make_https_server(self, certfile):
+            server = HTTPSServerThread(certfile)
+            flag = threading.Event()
+            server.start(flag)
+            flag.wait()
+            def cleanup():
+                server.stop()
+                server.join()
+            self.addCleanup(cleanup)
+            return server
 
-    def test_ssl_verification(self):
-        certfile = os.path.join(HERE, 'keycert.pem')
-        server = self.make_https_server(certfile)
-        url = 'https://localhost:%d/' % server.port
-        req = Request(url)
-        self.index.ssl_verifier = HTTPSHandler(certfile)
-        response = self.index.send_request(req)
-        self.assertEqual(response.code, 200)
+        def test_ssl_verification(self):
+            certfile = os.path.join(HERE, 'keycert.pem')
+            server = self.make_https_server(certfile)
+            url = 'https://localhost:%d/' % server.port
+            req = Request(url)
+            self.index.ssl_verifier = HTTPSHandler(certfile)
+            response = self.index.send_request(req)
+            self.assertEqual(response.code, 200)
 
-    def test_download(self):
-        digest = '913093474942c5a564c011f232868517' # for testsrc/README.txt
-        certfile = os.path.join(HERE, 'keycert.pem')
-        server = self.make_https_server(certfile)
-        url = 'https://localhost:%d/README.txt' % server.port
-        fd, fn = tempfile.mkstemp()
-        os.close(fd)
-        self.addCleanup(os.remove, fn)
-        with open(os.path.join(HERE, 'testsrc', 'README.txt'), 'rb') as f:
-            data = f.read()
-        self.index.ssl_verifier = HTTPSHandler(certfile)
-        self.index.download_file(url, fn)   # no digest
-        with open(fn, 'rb') as f:
-            self.assertEqual(data, f.read())
-        self.index.download_file(url, fn, digest)
-        with open(fn, 'rb') as f:
-            self.assertEqual(data, f.read())
-        reporthook = lambda *args: None
-        self.index.download_file(url, fn, ('md5', digest), reporthook)
-        with open(fn, 'rb') as f:
-            self.assertEqual(data, f.read())
-        # bad digest
-        self.assertRaises(DistlibException, self.index.download_file, url, fn,
-                          digest[:-1] + '8')
+        @unittest.skipIf(IN_GITHUB_WORKFLOW, 'This test is end-of-line dependent')
+        def test_download(self):  # pragma: no cover
+            digest = '913093474942c5a564c011f232868517' # for testsrc/README.txt
+            certfile = os.path.join(HERE, 'keycert.pem')
+            server = self.make_https_server(certfile)
+            url = 'https://localhost:%d/README.txt' % server.port
+            fd, fn = tempfile.mkstemp()
+            os.close(fd)
+            self.addCleanup(os.remove, fn)
+            with open(os.path.join(HERE, 'testsrc', 'README.txt'), 'rb') as f:
+                data = f.read()
+            self.index.ssl_verifier = HTTPSHandler(certfile)
+            self.index.download_file(url, fn)   # no digest
+            with open(fn, 'rb') as f:
+                self.assertEqual(data, f.read())
+            self.index.download_file(url, fn, digest)
+            with open(fn, 'rb') as f:
+                self.assertEqual(data, f.read())
+            reporthook = lambda *args: None
+            self.index.download_file(url, fn, ('md5', digest), reporthook)
+            with open(fn, 'rb') as f:
+                self.assertEqual(data, f.read())
+            # bad digest
+            self.assertRaises(DistlibException, self.index.download_file, url, fn,
+                              digest[:-1] + '8')
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
-    def test_search(self):
+    @unittest.skipUnless(ssl, 'SSL required for this test.')
+    @unittest.skipIf(True, 'skipping due to temporary changes in PyPI')
+    def test_search(self):  # pragma: no cover
         self.index = PackageIndex()
-        result = self.index.search({'name': 'tatterdema'})
+        result = self.index.search({'name': 'tatterdemalion'})
         self.assertEqual(len(result), 1)
         result = self.index.search({'name': 'ragamuff'})
-        self.assertEqual(len(result), 0)
+        if result:
+            msg = 'got an unexpected result: %s' % result
+        else:
+            msg = None
+        self.assertEqual(len(result), 0, msg)
 
 
 if __name__ == '__main__':  # pragma: no cover
