@@ -6,6 +6,7 @@
 #
 from __future__ import unicode_literals
 import os
+import posixpath
 try:
     import ssl
 except ImportError:
@@ -13,49 +14,52 @@ except ImportError:
 import sys
 
 from compat import unittest
+from support import DistlibTestCase
 
-from distlib.compat import url2pathname, urlparse, urljoin
+from distlib.compat import url2pathname, urlparse
 from distlib.database import (Distribution, DistributionPath, make_graph,
                               make_dist)
 from distlib.locators import (SimpleScrapingLocator, PyPIRPCLocator,
                               PyPIJSONLocator, DirectoryLocator,
                               DistPathLocator, AggregatingLocator,
-                              JSONLocator, DistPathLocator,
-                              DependencyFinder, locate,
+                              JSONLocator, DependencyFinder, locate,
                               get_all_distribution_names, default_locator)
 
 HERE = os.path.abspath(os.path.dirname(__file__))
 
 PYPI_RPC_HOST = 'http://python.org/pypi'
 
-PYPI_WEB_HOST = os.environ.get('PYPI_WEB_HOST', 'https://pypi.python.org/simple/')
+PYPI_WEB_HOST = os.environ.get('PYPI_WEB_HOST', 'https://pypi.org/simple/')
 
-class LocatorTestCase(unittest.TestCase):
+class LocatorTestCase(DistlibTestCase):
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(ssl, 'SSL required for this test.')
     def test_xmlrpc(self):
         locator = PyPIRPCLocator(PYPI_RPC_HOST)
         try:
-            result = locator.get_project('sarge')
-        except Exception:     # pragma: no cover
-            raise unittest.SkipTest('PyPI XML-RPC not available')
-        self.assertIn('0.1', result)
-        dist = result['0.1']
-        self.assertEqual(dist.name, 'sarge')
-        self.assertEqual(dist.version, '0.1')
-        possible = ('https://pypi.python.org/packages/source/s/sarge/'
-                    'sarge-0.1.tar.gz',
-                    'http://pypi.python.org/packages/source/s/sarge/'
-                    'sarge-0.1.tar.gz')
-        self.assertIn(dist.source_url, possible)
-        self.assertEqual(dist.digest,
-                         ('md5', '961ddd9bc085fdd8b248c6dd96ceb1c8'))
-        try:
-            names = locator.get_distribution_names()
-        except Exception:   # pragma: no cover
-            raise unittest.SkipTest('PyPI XML-RPC not available')
-        self.assertGreater(len(names), 25000)
+            try:
+                result = locator.get_project('sarge')
+            except Exception:     # pragma: no cover
+                raise unittest.SkipTest('PyPI XML-RPC not available')
+            self.assertIn('0.1', result)
+            dist = result['0.1']
+            self.assertEqual(dist.name, 'sarge')
+            self.assertEqual(dist.version, '0.1')
+            possible = ('https://pypi.org/packages/source/s/sarge/'
+                        'sarge-0.1.tar.gz',
+                        'http://pypi.org/packages/source/s/sarge/'
+                        'sarge-0.1.tar.gz')
+            self.assertIn(dist.source_url, possible)
+            self.assertEqual(dist.digest,
+                             ('md5', '961ddd9bc085fdd8b248c6dd96ceb1c8'))
+            try:
+                names = locator.get_distribution_names()
+            except Exception:   # pragma: no cover
+                raise unittest.SkipTest('PyPI XML-RPC not available')
+            self.assertGreater(len(names), 25000)
+        finally:
+            locator.client('close')()
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(ssl, 'SSL required for this test.')
@@ -63,7 +67,10 @@ class LocatorTestCase(unittest.TestCase):
         locator = PyPIJSONLocator(PYPI_RPC_HOST)
         result = locator.get_project('sarge')
         LATEST_SARGE_VERSION = '0.1.4'
-        LATEST_SARGE_MD5 = '285013875aa908ef1417055d3e74a00a'
+        LATEST_SARGE_HASHES = (
+            ('md5', '285013875aa908ef1417055d3e74a00a'),
+            ('sha256', '59f93216723ddd9062d17cbbb90ed9e69267b84825cf0bde0b7f8d934c424823'),
+        )
         self.assertIn(LATEST_SARGE_VERSION, result)
         dist = result[LATEST_SARGE_VERSION]
         self.assertEqual(dist.name, 'sarge')
@@ -71,13 +78,13 @@ class LocatorTestCase(unittest.TestCase):
         path = '/sarge-%s.tar.gz' % LATEST_SARGE_VERSION
         for url in dist.download_urls:
             self.assertTrue(url.endswith(path))
-        self.assertEqual(dist.digests[url],('md5', LATEST_SARGE_MD5))
+        self.assertIn(dist.digests[url], LATEST_SARGE_HASHES)
         self.assertRaises(NotImplementedError, locator.get_distribution_names)
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(ssl, 'SSL required for this test.')
     def test_scraper(self):
-        locator = SimpleScrapingLocator('https://pypi.python.org/simple/')
+        locator = SimpleScrapingLocator('https://pypi.org/simple/')
         for name in ('sarge', 'Sarge'):
             result = locator.get_project(name)
             self.assertIn('0.1', result)
@@ -85,8 +92,24 @@ class LocatorTestCase(unittest.TestCase):
             self.assertEqual(dist.name, 'sarge')
             self.assertEqual(dist.version, '0.1')
             self.assertTrue(dist.source_url.endswith('/sarge-0.1.tar.gz'))
-            self.assertEqual(dist.digest,
-                             ('md5', '961ddd9bc085fdd8b248c6dd96ceb1c8'))
+            SARGE_HASHES = (
+                ('md5', '961ddd9bc085fdd8b248c6dd96ceb1c8'),
+                ('sha256', 'ec2ec0b1c9ed9a77f9b4322c16e4954c93aa00d974a1af931b18eb751e377dfe'),
+            )
+            self.assertIn(dist.digest, SARGE_HASHES)
+        # Test to check issue #112 fix.
+        locator.wheel_tags = [('cp27', 'cp27m', 'win_amd64'),
+                              ('cp35', 'cp35m', 'win32')]
+        result = locator.get_project('simplejson')
+        urls = result['urls'].get('3.16.0')
+        self.assertTrue(urls)
+        self.assertEqual(3, len(urls))
+        expected = set(['simplejson-3.16.0-cp27-cp27m-win_amd64.whl',
+                        'simplejson-3.16.0-cp35-cp35m-win32.whl',
+                        'simplejson-3.16.0.tar.gz'])
+        for u in urls:
+            p = posixpath.split(urlparse(u).path)[-1]
+            self.assertIn(p, expected)
         return
         # The following is too slow
         names = locator.get_distribution_names()
@@ -97,11 +120,11 @@ class LocatorTestCase(unittest.TestCase):
     def test_unicode_project_name(self):
         # Just checking to see that no exceptions are raised.
         NAME = '\u2603'
-        locator = SimpleScrapingLocator('https://pypi.python.org/simple/')
+        locator = SimpleScrapingLocator('https://pypi.org/simple/')
         result = locator.get_project(NAME)
         expected = {'urls': {}, 'digests': {}}
         self.assertEqual(result, expected)
-        locator = PyPIJSONLocator('https://pypi.python.org/pypi/')
+        locator = PyPIJSONLocator('https://pypi.org/pypi/')
         result = locator.get_project(NAME)
         self.assertEqual(result, expected)
 
@@ -176,7 +199,7 @@ class LocatorTestCase(unittest.TestCase):
     def test_aggregation(self):
         d = os.path.join(HERE, 'fake_archives')
         loc1 = DirectoryLocator(d)
-        loc2 = SimpleScrapingLocator('https://pypi.python.org/simple/',
+        loc2 = SimpleScrapingLocator('https://pypi.org/simple/',
                                      timeout=5.0)
         locator = AggregatingLocator(loc1, loc2)
         exp1 = os.path.join(HERE, 'fake_archives', 'subdir',
@@ -213,8 +236,7 @@ class LocatorTestCase(unittest.TestCase):
     def test_dependency_finder(self):
         locator = AggregatingLocator(
             JSONLocator(),
-            SimpleScrapingLocator('https://pypi.python.org/simple/',
-                                  timeout=3.0),
+            SimpleScrapingLocator('https://pypi.org/simple/', timeout=3.0),
             scheme='legacy')
         finder = DependencyFinder(locator)
         dists, problems = finder.find('irc (== 5.0.1)')
@@ -226,33 +248,92 @@ class LocatorTestCase(unittest.TestCase):
                                       meta_extras=[':test:'])
         self.assertFalse(problems)
         actual = sorted([d.name for d in dists])
-        self.assertEqual(actual, ['hgtools', 'irc', 'py',
-                                  'pytest',
-                                  'pytest-runner', 'setuptools_scm'])
-
+        # expected = [
+            # 'atomicwrites',
+            # 'attrs',
+            # 'colorama',
+            # 'hgtools',
+            # 'importlib-metadata',
+            # 'irc',
+            # 'more-itertools',
+            # 'packaging',
+            # 'pathlib2',
+            # 'pluggy',
+            # 'py',
+            # 'pyparsing',
+            # 'pytest',
+            # 'pytest-runner',
+            # 'setuptools',
+            # 'setuptools-scm',
+            # 'setuptools_scm',
+            # 'six',
+            # 'wcwidth'
+        # ]
+        expected = ['hgtools', 'irc', 'pytest', 'pytest-runner', 'setuptools_scm']
+        self.assertEqual(actual, expected)
         g = make_graph(dists)
         slist, cycle = g.topological_sort()
         self.assertFalse(cycle)
         names = [d.name for d in slist]
         expected = set([
-            ('setuptools_scm', 'hgtools', 'py', 'pytest-runner', 'pytest', 'irc'),
-            ('setuptools_scm', 'hgtools', 'py', 'pytest', 'pytest-runner', 'irc'),
-            ('setuptools_scm', 'py', 'hgtools', 'pytest-runner', 'pytest', 'irc'),
-            ('hgtools', 'setuptools_scm', 'py', 'pytest', 'pytest-runner', 'irc'),
-            ('py', 'hgtools', 'setuptools_scm', 'pytest', 'pytest-runner', 'irc'),
-            ('hgtools', 'setuptools_scm', 'py', 'pytest-runner', 'pytest', 'irc'),
-            ('py', 'hgtools', 'setuptools_scm', 'pytest-runner', 'pytest', 'irc'),
-            ('py', 'setuptools_scm', 'hgtools', 'pytest', 'pytest-runner', 'irc'),
+            ('atomicwrites', 'six', 'importlib-metadata', 'more-itertools',
+             'attrs', 'pyparsing', 'setuptools', 'setuptools_scm',
+             'setuptools-scm', 'wcwidth', 'colorama', 'py', 'pathlib2',
+             'pluggy', 'hgtools', 'pytest-runner', 'packaging', 'irc',
+             'pytest'),
+            ('atomicwrites', 'setuptools', 'importlib-metadata',
+            'more-itertools', 'attrs', 'wcwidth', 'pyparsing', 'colorama',
+            'six', 'setuptools_scm', 'setuptools-scm', 'pluggy', 'packaging',
+            'py', 'hgtools', 'pytest-runner', 'pathlib2', 'irc', 'pytest'),
+            ('wcwidth', 'setuptools_scm', 'colorama', 'six',
+            'importlib-metadata', 'setuptools', 'setuptools-scm', 'pyparsing',
+            'more-itertools', 'attrs', 'atomicwrites', 'pathlib2',
+            'packaging', 'pytest-runner', 'hgtools', 'py', 'pluggy', 'irc',
+            'pytest'),
+            ('setuptools', 'setuptools-scm', 'wcwidth',
+            'importlib-metadata', 'colorama', 'six', 'atomicwrites',
+            'more-itertools', 'attrs', 'pyparsing', 'setuptools_scm',
+            'pluggy', 'pathlib2', 'packaging', 'py', 'pytest-runner',
+            'hgtools', 'irc', 'pytest'),
+            ('atomicwrites', 'setuptools', 'importlib-metadata', 'attrs',
+             'wcwidth', 'more-itertools', 'pyparsing', 'colorama', 'six',
+             'setuptools_scm', 'setuptools-scm', 'pluggy', 'packaging',
+             'hgtools', 'pytest-runner', 'pathlib2', 'py', 'irc', 'pytest'),
+            ('wcwidth', 'setuptools_scm', 'more-itertools', 'colorama', 'six',
+             'importlib-metadata', 'setuptools', 'setuptools-scm',
+             'atomicwrites', 'pyparsing', 'attrs', 'pathlib2', 'packaging',
+             'pytest-runner', 'py', 'pluggy', 'hgtools', 'irc', 'pytest'),
+            ('six', 'atomicwrites', 'setuptools', 'setuptools-scm', 'attrs',
+             'importlib-metadata', 'more-itertools', 'wcwidth', 'colorama', 'pyparsing',
+             'setuptools_scm', 'pluggy', 'packaging', 'hgtools', 'py', 'pytest-runner',
+             'pathlib2', 'irc', 'pytest'),
+            ('wcwidth', 'setuptools_scm', 'more-itertools', 'six', 'setuptools',
+             'importlib-metadata', 'pyparsing', 'setuptools-scm', 'atomicwrites',
+             'colorama', 'attrs', 'pathlib2', 'packaging', 'pytest-runner', 'hgtools',
+             'py', 'pluggy', 'pytest', 'irc'),
+            ('atomicwrites', 'importlib-metadata', 'attrs', 'pyparsing', 'wcwidth',
+             'colorama', 'setuptools', 'more-itertools', 'six', 'setuptools_scm',
+             'setuptools-scm', 'pluggy', 'packaging', 'hgtools', 'pytest-runner',
+             'pathlib2', 'py', 'irc', 'pytest'),
+            ('atomicwrites', 'pyparsing', 'six', 'setuptools', 'importlib-metadata',
+             'more-itertools', 'attrs', 'setuptools_scm', 'setuptools-scm', 'wcwidth',
+             'colorama', 'py', 'pathlib2', 'pluggy', 'hgtools', 'pytest-runner',
+             'packaging', 'irc', 'pytest'),
+            ('atomicwrites', 'setuptools', 'importlib-metadata', 'more-itertools',
+             'attrs', 'pyparsing', 'wcwidth', 'colorama', 'six', 'setuptools_scm',
+             'setuptools-scm', 'pluggy', 'packaging', 'py', 'hgtools', 'pytest-runner',
+             'pathlib2', 'irc', 'pytest'),
+            ('wcwidth', 'setuptools_scm', 'more-itertools', 'six', 'setuptools',
+             'importlib-metadata', 'pyparsing', 'setuptools-scm', 'atomicwrites',
+             'attrs', 'colorama', 'pathlib2', 'packaging', 'pytest-runner', 'hgtools',
+             'py', 'pluggy', 'pytest', 'irc'),
+            ('six', 'atomicwrites', 'setuptools', 'setuptools-scm', 'attrs',
+             'importlib-metadata', 'more-itertools', 'wcwidth', 'pyparsing', 'colorama',
+             'setuptools_scm', 'pluggy', 'packaging', 'hgtools', 'py', 'pytest-runner',
+             'pathlib2', 'irc', 'pytest'),
             ('pytest', 'setuptools_scm', 'hgtools', 'pytest-runner', 'irc'),
-            ('hgtools', 'setuptools_scm', 'pytest', 'pytest-runner', 'irc'),
-            ('py', 'setuptools_scm', 'hgtools', 'pytest-runner', 'pytest', 'irc'),
-            ('py', 'setuptools_scm', 'pytest', 'pytest-runner', 'hgtools', 'irc'),
-            ('py', 'setuptools_scm', 'pytest-runner', 'pytest', 'hgtools', 'irc'),
-            ('py', 'setuptools_scm', 'pytest', 'hgtools', 'pytest-runner', 'irc'),
-            ('setuptools_scm', 'py', 'pytest', 'hgtools', 'pytest-runner', 'irc'),
-            ('setuptools_scm', 'py', 'pytest-runner', 'hgtools', 'pytest', 'irc'),
-            ('py', 'setuptools_scm', 'pytest-runner', 'hgtools', 'pytest', 'irc'),
-            ('setuptools_scm', 'py', 'pytest', 'pytest-runner', 'hgtools', 'irc'),
+            ('pytest', 'setuptools_scm', 'pytest-runner', 'hgtools', 'irc'),
+            ('setuptools_scm', 'pytest', 'hgtools', 'pytest-runner', 'irc'),
         ])
         self.assertIn(tuple(names), expected)
 
@@ -273,7 +354,7 @@ class LocatorTestCase(unittest.TestCase):
         # Now test with extra in dependency
         locator.clear_cache()
         dummy = make_dist('dummy', '0.1')
-        dummy.metadata.run_requires = [{'requires': ['Jinja2 [i18n]']}]
+        dummy.metadata.run_requires = [{'requires': ['Jinja2 [i18n] (<2.8)']}]
         dists, problems = finder.find(dummy)
         self.assertFalse(problems)
         actual = sorted([d.name_and_version for d in dists])
@@ -296,8 +377,8 @@ class LocatorTestCase(unittest.TestCase):
             self.assertGreater(len(all_dists), 0)
 
     def test_url_preference(self):
-        cases = (('http://netloc/path', 'https://netloc/path'),
-                 ('http://pypi.python.org/path', 'http://netloc/path'),
+        cases = (('https://netloc/path', 'http://netloc/path'),
+                 ('http://pypi.org/path', 'http://netloc/path'),
                  ('http://netloc/B', 'http://netloc/A'))
         for url1, url2 in cases:
             self.assertEqual(default_locator.prefer_url(url1, url2), url1)
@@ -307,7 +388,7 @@ class LocatorTestCase(unittest.TestCase):
     def test_prereleases(self):
         locator = AggregatingLocator(
             JSONLocator(),
-            SimpleScrapingLocator('https://pypi.python.org/simple/',
+            SimpleScrapingLocator('https://pypi.org/simple/',
                                   timeout=3.0),
             scheme='legacy')
         REQT = 'SQLAlchemy (>0.5.8, < 0.6)'
@@ -335,17 +416,17 @@ class LocatorTestCase(unittest.TestCase):
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(ssl, 'SSL required for this test.')
     def test_dist_reqts(self):
-        r = 'config (<=0.3.5)'
+        r = 'config <=0.3.9'
         dist = default_locator.locate(r)
         self.assertIsNotNone(dist)
         self.assertIsNone(dist.extras)
         self.assertTrue(dist.matches_requirement(r))
-        self.assertFalse(dist.matches_requirement('config (0.3.6)'))
+        self.assertFalse(dist.matches_requirement('config == 0.4.0'))
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(ssl, 'SSL required for this test.')
     def test_dist_reqts_extras(self):
-        r = 'config[doc,test](<=0.3.5)'
+        r = 'config[doc,test](<=0.3.9)'
         dist = default_locator.locate(r)
         self.assertIsNotNone(dist)
         self.assertTrue(dist.matches_requirement(r))
@@ -353,7 +434,7 @@ class LocatorTestCase(unittest.TestCase):
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(ssl, 'SSL required for this test.')
-    def test_all(self):
+    def test_all_versions(self):
         d = default_locator.get_project('setuptools')
         self.assertTrue('urls' in d)
         d = d['urls']
@@ -490,8 +571,6 @@ class LocatorTestCase(unittest.TestCase):
             'setuptools-6.0.1.zip',
             'setuptools-6.0.2.tar.gz',
             'setuptools-6.0.2.zip',
-            # 'setuptools-6.0.tar.gz',
-            'setuptools-6.0.zip',
         ])
         actual = set()
         for k, v in d.items():
@@ -510,6 +589,6 @@ class LocatorTestCase(unittest.TestCase):
 
 if __name__ == '__main__':  # pragma: no cover
     import logging
-    logging.basicConfig(level=logging.DEBUG, filename='test_locators.log',
+    logging.basicConfig(level=logging.DEBUG, filename='run/test_locators.log',
                         filemode='w', format='%(message)s')
     unittest.main()
