@@ -6,8 +6,7 @@
 #
 from __future__ import unicode_literals
 
-import codecs
-import hashlib
+import io
 import os
 import re
 import shutil
@@ -16,16 +15,17 @@ import sys
 import tempfile
 
 from compat import unittest
+from support import DistlibTestCase
 
 from distlib import DistlibException
 from distlib.compat import ZipFile, sysconfig, fsencode
-from distlib.database import DistributionPath, InstalledDistribution
+from distlib.database import DistributionPath
 from distlib.manifest import Manifest
-from distlib.metadata import Metadata, METADATA_FILENAME
+from distlib.metadata import Metadata, METADATA_FILENAME, LEGACY_METADATA_FILENAME
 from distlib.scripts import ScriptMaker
 from distlib.util import get_executable
 from distlib.wheel import (Wheel, PYVER, IMPVER, ARCH, ABI, COMPATIBLE_TAGS,
-                           is_compatible)
+                           IMP_PREFIX, is_compatible, _get_glibc_version)
 
 try:
     with open(os.devnull, 'wb') as junk:
@@ -39,6 +39,29 @@ HERE = os.path.abspath(os.path.dirname(__file__))
 
 EGG_INFO_RE = re.compile(r'(-py\d\.\d)?\.egg-info', re.I)
 
+def pip_version():
+    result = None
+    fd, fn = tempfile.mkstemp(prefix='distlib-test-', suffix='.txt')
+    try:
+        os.close(fd)
+        with open(fn, 'wb') as out:
+            subprocess.check_call(['pip', '--version'], stdout=out,
+                                   stderr=subprocess.STDOUT)
+        with io.open(fn, encoding='utf-8') as f:
+            data = f.read().split()
+        assert data[0] == 'pip'
+        parts = data[1].split('.')
+        result = []
+        for p in parts:
+            if p.isdigit():
+                result.append(int(p))
+            else:
+                result.append(p)
+        result = tuple(result)
+    finally:
+        os.remove(fn)
+    return result
+
 def convert_egg_info(libdir, prefix):
     files = os.listdir(libdir)
     ei = list(filter(lambda d: d.endswith('.egg-info'), files))[0]
@@ -68,7 +91,7 @@ def install_dist(distname, workdir):
     scripts = pfx + '--install-scripts=%s/scripts' % workdir
     data = pfx + '--install-data=%s/data' % workdir
     cmd = ['pip', 'install',
-           '--index-url', 'https://pypi.python.org/simple/',
+           '--index-url', 'https://pypi.org/simple/',
            '--timeout', '3', '--default-timeout', '3',
            purelib, platlib, headers, scripts, data, distname]
     result = {
@@ -95,7 +118,7 @@ def install_dist(distname, workdir):
     return result
 
 
-class WheelTestCase(unittest.TestCase):
+class WheelTestCase(DistlibTestCase):
 
     def test_valid_filename(self):
         attrs = ('name', 'version', 'buildver', 'pyver', 'abi', 'arch')
@@ -164,12 +187,189 @@ class WheelTestCase(unittest.TestCase):
         self.assertIn((PYVER[:-1], 'none', 'any'), tags)
         this_arch = filter(lambda o: o[-1] == ARCH, tags)
         self.assertTrue(this_arch)
+        if sys.platform.startswith('linux'):
+            arch = ARCH.replace('linux_', '')
+            parts = _get_glibc_version()
+            if len(parts) == 2:
+                self.assertTrue(filter(lambda o: o[-1] == 'manylinux_%s_%s_%s' %
+                                       (parts[0], parts[1], arch), tags))
+                if parts >= (2, 17):
+                    self.assertTrue(filter(lambda o: o[-1] == 'manylinux2014_%s' %
+                                           arch, tags))
+                if parts >= (2, 12):
+                    self.assertTrue(filter(lambda o: o[-1] == 'manylinux2010_%s' %
+                                           arch, tags))
+                if parts >= (2, 5):
+                    self.assertTrue(filter(lambda o: o[-1] == 'manylinux1_%s' %
+                                           arch, tags))
+
 
     def test_is_compatible(self):
         fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
-        if 'py27' <= PYVER < 'py32':
+        if PYVER in ('py27', 'py30', 'py31'):
             self.assertTrue(is_compatible(fn))
             self.assertTrue(Wheel(fn).is_compatible())
+        # use actual wheel names from PyPI.
+        wheel_names = [
+            'simplejson-3.17.2-cp27-cp27m-macosx_10_13_x86_64.whl',
+            'simplejson-3.17.2-cp27-cp27m-manylinux1_i686.whl',
+            'simplejson-3.17.2-cp27-cp27m-manylinux1_x86_64.whl',
+            'simplejson-3.17.2-cp27-cp27m-manylinux2010_i686.whl',
+            'simplejson-3.17.2-cp27-cp27m-manylinux2010_x86_64.whl',
+            'simplejson-3.17.2-cp27-cp27mu-manylinux1_i686.whl',
+            'simplejson-3.17.2-cp27-cp27mu-manylinux1_x86_64.whl',
+            'simplejson-3.17.2-cp27-cp27mu-manylinux2010_i686.whl',
+            'simplejson-3.17.2-cp27-cp27mu-manylinux2010_x86_64.whl',
+            'simplejson-3.17.2-cp27-cp27m-win32.whl',
+            'simplejson-3.17.2-cp27-cp27m-win_amd64.whl',
+            'simplejson-3.17.2-cp33-cp33m-win32.whl',
+            'simplejson-3.17.2-cp33-cp33m-win_amd64.whl',
+            'simplejson-3.17.2-cp34-cp34m-win32.whl',
+            'simplejson-3.17.2-cp34-cp34m-win_amd64.whl',
+            'simplejson-3.17.2-cp35-cp35m-manylinux1_i686.whl',
+            'simplejson-3.17.2-cp35-cp35m-manylinux1_x86_64.whl',
+            'simplejson-3.17.2-cp35-cp35m-manylinux2010_i686.whl',
+            'simplejson-3.17.2-cp35-cp35m-manylinux2010_x86_64.whl',
+            'simplejson-3.17.2-cp35-cp35m-manylinux2014_aarch64.whl',
+            'simplejson-3.17.2-cp35-cp35m-win32.whl',
+            'simplejson-3.17.2-cp35-cp35m-win_amd64.whl',
+            'simplejson-3.17.2-cp36-cp36m-macosx_10_13_x86_64.whl',
+            'simplejson-3.17.2-cp36-cp36m-manylinux1_i686.whl',
+            'simplejson-3.17.2-cp36-cp36m-manylinux1_x86_64.whl',
+            'simplejson-3.17.2-cp36-cp36m-manylinux2010_i686.whl',
+            'simplejson-3.17.2-cp36-cp36m-manylinux2010_x86_64.whl',
+            'simplejson-3.17.2-cp36-cp36m-manylinux2014_aarch64.whl',
+            'simplejson-3.17.2-cp36-cp36m-win32.whl',
+            'simplejson-3.17.2-cp36-cp36m-win_amd64.whl',
+            'simplejson-3.17.2-cp37-cp37m-macosx_10_14_x86_64.whl',
+            'simplejson-3.17.2-cp37-cp37m-manylinux1_i686.whl',
+            'simplejson-3.17.2-cp37-cp37m-manylinux1_x86_64.whl',
+            'simplejson-3.17.2-cp37-cp37m-manylinux2010_i686.whl',
+            'simplejson-3.17.2-cp37-cp37m-manylinux2010_x86_64.whl',
+            'simplejson-3.17.2-cp37-cp37m-manylinux2014_aarch64.whl',
+            'simplejson-3.17.2-cp37-cp37m-win32.whl',
+            'simplejson-3.17.2-cp37-cp37m-win_amd64.whl',
+            'simplejson-3.17.2-cp38-cp38-macosx_10_14_x86_64.whl',
+            'simplejson-3.17.2-cp38-cp38-manylinux1_i686.whl',
+            'simplejson-3.17.2-cp38-cp38-manylinux1_x86_64.whl',
+            'simplejson-3.17.2-cp38-cp38-manylinux2010_i686.whl',
+            'simplejson-3.17.2-cp38-cp38-manylinux2010_x86_64.whl',
+            'simplejson-3.17.2-cp38-cp38-manylinux2014_aarch64.whl',
+
+            'Pillow-7.2.0-cp35-cp35m-macosx_10_10_intel.whl',
+            'Pillow-7.2.0-cp35-cp35m-manylinux1_i686.whl',
+            'Pillow-7.2.0-cp35-cp35m-manylinux1_x86_64.whl',
+            'Pillow-7.2.0-cp35-cp35m-manylinux2014_aarch64.whl',
+            'Pillow-7.2.0-cp35-cp35m-win32.whl',
+            'Pillow-7.2.0-cp35-cp35m-win_amd64.whl',
+            'Pillow-7.2.0-cp36-cp36m-macosx_10_10_x86_64.whl',
+            'Pillow-7.2.0-cp36-cp36m-manylinux1_i686.whl',
+            'Pillow-7.2.0-cp36-cp36m-manylinux1_x86_64.whl',
+            'Pillow-7.2.0-cp36-cp36m-manylinux2014_aarch64.whl',
+            'Pillow-7.2.0-cp36-cp36m-win32.whl',
+            'Pillow-7.2.0-cp36-cp36m-win_amd64.whl',
+            'Pillow-7.2.0-cp37-cp37m-macosx_10_10_x86_64.whl',
+            'Pillow-7.2.0-cp37-cp37m-manylinux1_i686.whl',
+            'Pillow-7.2.0-cp37-cp37m-manylinux1_x86_64.whl',
+            'Pillow-7.2.0-cp37-cp37m-manylinux2014_aarch64.whl',
+            'Pillow-7.2.0-cp37-cp37m-win32.whl',
+            'Pillow-7.2.0-cp37-cp37m-win_amd64.whl',
+            'Pillow-7.2.0-cp38-cp38-macosx_10_10_x86_64.whl',
+            'Pillow-7.2.0-cp38-cp38-manylinux1_i686.whl',
+            'Pillow-7.2.0-cp38-cp38-manylinux1_x86_64.whl',
+            'Pillow-7.2.0-cp38-cp38-manylinux2014_aarch64.whl',
+            'Pillow-7.2.0-cp38-cp38-win32.whl',
+            'Pillow-7.2.0-cp38-cp38-win_amd64.whl',
+            'Pillow-7.2.0-pp36-pypy36_pp73-macosx_10_10_x86_64.whl',
+            'Pillow-7.2.0-pp36-pypy36_pp73-manylinux2010_x86_64.whl',
+            'Pillow-7.2.0-pp36-pypy36_pp73-win32.whl',
+
+            'reportlab-3.5.47-cp27-cp27m-macosx_10_9_x86_64.whl',
+            'reportlab-3.5.47-cp27-cp27m-manylinux1_i686.whl',
+            'reportlab-3.5.47-cp27-cp27m-manylinux1_x86_64.whl',
+            'reportlab-3.5.47-cp27-cp27m-manylinux2010_i686.whl',
+            'reportlab-3.5.47-cp27-cp27m-manylinux2010_x86_64.whl',
+            'reportlab-3.5.47-cp27-cp27mu-manylinux1_i686.whl',
+            'reportlab-3.5.47-cp27-cp27mu-manylinux1_x86_64.whl',
+            'reportlab-3.5.47-cp27-cp27mu-manylinux2010_i686.whl',
+            'reportlab-3.5.47-cp27-cp27mu-manylinux2010_x86_64.whl',
+            'reportlab-3.5.47-cp27-cp27m-win32.whl',
+            'reportlab-3.5.47-cp27-cp27m-win_amd64.whl',
+            'reportlab-3.5.47-cp35-cp35m-macosx_10_6_intel.whl',
+            'reportlab-3.5.47-cp35-cp35m-manylinux1_i686.whl',
+            'reportlab-3.5.47-cp35-cp35m-manylinux1_x86_64.whl',
+            'reportlab-3.5.47-cp35-cp35m-manylinux2010_i686.whl',
+            'reportlab-3.5.47-cp35-cp35m-manylinux2010_x86_64.whl',
+            'reportlab-3.5.47-cp35-cp35m-win32.whl',
+            'reportlab-3.5.47-cp35-cp35m-win_amd64.whl',
+            'reportlab-3.5.47-cp36-cp36m-macosx_10_9_x86_64.whl',
+            'reportlab-3.5.47-cp36-cp36m-manylinux1_i686.whl',
+            'reportlab-3.5.47-cp36-cp36m-manylinux1_x86_64.whl',
+            'reportlab-3.5.47-cp36-cp36m-manylinux2010_i686.whl',
+            'reportlab-3.5.47-cp36-cp36m-manylinux2010_x86_64.whl',
+            'reportlab-3.5.47-cp36-cp36m-win32.whl',
+            'reportlab-3.5.47-cp36-cp36m-win_amd64.whl',
+            'reportlab-3.5.47-cp37-cp37m-macosx_10_9_x86_64.whl',
+            'reportlab-3.5.47-cp37-cp37m-manylinux1_i686.whl',
+            'reportlab-3.5.47-cp37-cp37m-manylinux1_x86_64.whl',
+            'reportlab-3.5.47-cp37-cp37m-manylinux2010_i686.whl',
+            'reportlab-3.5.47-cp37-cp37m-manylinux2010_x86_64.whl',
+            'reportlab-3.5.47-cp37-cp37m-win32.whl',
+            'reportlab-3.5.47-cp37-cp37m-win_amd64.whl',
+            'reportlab-3.5.47-cp38-cp38-macosx_10_9_x86_64.whl',
+            'reportlab-3.5.47-cp38-cp38-manylinux1_i686.whl',
+            'reportlab-3.5.47-cp38-cp38-manylinux1_x86_64.whl',
+            'reportlab-3.5.47-cp38-cp38-manylinux2010_i686.whl',
+            'reportlab-3.5.47-cp38-cp38-manylinux2010_x86_64.whl',
+            'reportlab-3.5.47-cp38-cp38-win32.whl',
+            'reportlab-3.5.47-cp38-cp38-win_amd64.whl',
+        ]
+        for fn in filter(is_compatible, wheel_names):
+            w = Wheel(fn)
+            our_arch = ARCH.replace('linux_', '')
+            for pyver, abi, arch in w.tags:
+                self.assertEqual(pyver, IMPVER)
+                self.assertEqual(abi, ABI)
+                if sys.platform != 'darwin':
+                    self.assertTrue(arch.endswith(our_arch))
+                else:
+                    m = re.match(r'(\w+)_(\d+)_(\d+)_(\w+)$', our_arch)
+                    self.assertTrue(m)
+                    _, major, minor, our_arch_kind = m.groups()
+                    our_major = int(major)
+                    our_minor = int(minor)
+                    m = re.match(r'(\w+)_(\d+)_(\d+)_(\w+)$', arch)
+                    self.assertTrue(m)
+                    _, major, minor, arch_kind = m.groups()
+                    major = int(major)
+                    minor = int(minor)
+                    self.assertEqual(major, our_major)
+                    self.assertLessEqual(minor, our_minor)
+                    if arch_kind in ('x86_64', 'i386'):
+                        self.assertEqual(arch_kind, our_arch_kind)
+                    elif arch_kind == 'fat':
+                        self.assertIn(our_arch_kind, ('i386', 'ppc'))
+                    elif arch_kind == 'fat3':
+                        self.assertIn(our_arch_kind, ('i386', 'ppc', 'x86_x64'))
+                    elif arch_kind == 'fat64':
+                        self.assertIn(our_arch_kind, ('ppc64', 'x86_x64'))
+                    elif arch_kind == 'intel':
+                        self.assertIn(our_arch_kind, ('i386', 'x86_x64'))
+                    elif arch_kind == 'universal':
+                        self.assertIn(our_arch_kind, ('i386', 'ppc', 'ppc64', 'x86_x64', 'intel'))
+                if 'manylinux' in arch:
+                    self.assertTrue(sys.platform.startswith('linux'))
+                    parts = _get_glibc_version()
+                    self.assertEqual(len(parts), 2)
+                    if 'manylinux2014_' in arch:
+                        self.assertTrue(parts >= (2, 17))
+                    if 'manylinux2010_' in arch:
+                        self.assertTrue(parts >= (2, 12))
+                    if 'manylinux1_' in arch:
+                        self.assertTrue(parts >= (2, 5))
+                    if 'manylinux_' in arch:
+                        s = 'manylinux_%s_%s_' % parts
+                        self.assertIn(s, arch)
 
     def test_metadata(self):
         fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
@@ -354,6 +554,10 @@ class WheelTestCase(unittest.TestCase):
         fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
         w = Wheel(fn)
         w.verify()
+        # see issue 115
+        fn = os.path.join(HERE, 'valid_wheel-0.0.1-py3-none-any.whl')
+        w = Wheel(fn)
+        w.verify()
         fn = os.path.join(HERE, 'bad_wheels', 'dummy-0.1-py27-none-any.whl')
         w = Wheel(fn)
         self.assertRaises(DistlibException, w.verify)
@@ -362,17 +566,17 @@ class WheelTestCase(unittest.TestCase):
         return False
 
     def wheel_modifier(self, path_map):
-        mdpath = path_map['dummy-0.1.dist-info/pydist.json']
+        mdpath = path_map['dummy-0.1.dist-info/%s' % LEGACY_METADATA_FILENAME]
         md = Metadata(path=mdpath)
         md.add_requirements(['numpy'])
-        md.write(path=mdpath)
+        md.write(path=mdpath, legacy=True)
         return True
 
     def wheel_modifier_ver(self, path_map):
-        mdpath = path_map['dummy-0.1.dist-info/pydist.json']
+        mdpath = path_map['dummy-0.1.dist-info/%s' % LEGACY_METADATA_FILENAME]
         md = Metadata(path=mdpath)
         md.version = '0.1+123'
-        md.write(path=mdpath)
+        md.write(path=mdpath, legacy=True)
         return True
 
     def test_update(self):
@@ -393,8 +597,8 @@ class WheelTestCase(unittest.TestCase):
         w = Wheel(dfn)
         w.verify()
         md = w.metadata
-        self.assertEqual(md.run_requires, [{'requires': ['numpy']}])
-        self.assertEquals(md.version, '0.1+1')
+        self.assertEqual(md.run_requires, ['numpy'])
+        self.assertEqual(md.version, '0.1+1')
 
         modified = w.update(self.wheel_modifier_ver)
         self.assertTrue(modified)
@@ -402,8 +606,8 @@ class WheelTestCase(unittest.TestCase):
         w = Wheel(dfn)
         w.verify()
         md = w.metadata
-        self.assertEqual(md.run_requires, [{'requires': ['numpy']}])
-        self.assertEquals(md.version, '0.1+123')
+        self.assertEqual(md.run_requires, ['numpy'])
+        self.assertEqual(md.version, '0.1+123')
 
     def test_info(self):
         fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
@@ -470,11 +674,19 @@ class WheelTestCase(unittest.TestCase):
         parts = ['cp', pyver]
         if sysconfig.get_config_var('Py_DEBUG'):
             parts.append('d')
-        if sysconfig.get_config_var('WITH_PYMALLOC'):
-            parts.append('m')
-        if sysconfig.get_config_var('Py_UNICODE_SIZE') == 4:
-            parts.append('u')
-        if sys.version_info[:2] < (3, 5):
+        # Starting with 3.8, the SOABI doesn't append m when WITH_PYMALLOC is
+        # defined (see bpo-36707)
+        vi = sys.version_info[:2]
+        if IMP_PREFIX == 'cp':
+            if vi < (3, 8):
+                wpm = sysconfig.get_config_var('WITH_PYMALLOC')
+                if wpm or wpm is None:
+                    parts.append('m')
+            if vi < (3, 3):
+                us = sysconfig.get_config_var('Py_UNICODE_SIZE')
+                if us == 4 or (us is None and sys.maxunicode == 0x10FFFF):
+                    parts.append('u')
+        if vi < (3, 5):
             abi = ABI
         else:
             abi = ABI.split('-')[0]
@@ -482,7 +694,11 @@ class WheelTestCase(unittest.TestCase):
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
+    @unittest.skipIf(sys.version_info[:2] >= (3, 7), 'The test distribution is not '
+                                               '3.7+ compatible')
     def test_build_and_install_pure(self):
+        if pip_version() >= (20, 2, 0):
+            raise unittest.SkipTest('Test not supported by pip version')
         self.do_build_and_install('sarge == 0.1')
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
@@ -492,6 +708,8 @@ class WheelTestCase(unittest.TestCase):
                                                'builds on Linux')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
     def test_build_and_install_plat(self):
+        if pip_version() >= (20, 2, 0):
+            raise unittest.SkipTest('Test not supported by pip version')
         self.do_build_and_install('hiredis == 0.1.1')
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
@@ -499,13 +717,17 @@ class WheelTestCase(unittest.TestCase):
                                                '3.x compatible')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
     def test_build_and_install_data(self):
-        self.do_build_and_install('Werkzeug == 0.4')
+        if pip_version() >= (20, 2, 0):
+            raise unittest.SkipTest('Test not supported by pip version')
+        self.do_build_and_install('Werkzeug == 0.5')
 
     @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipIf(sys.version_info[0] == 3, 'The test distribution is not '
                                                '3.x compatible')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
     def test_build_and_install_scripts(self):
+        if pip_version() >= (20, 2, 0):
+            raise unittest.SkipTest('Test not supported by pip version')
         self.do_build_and_install('Babel == 0.9.6')
 
 if __name__ == '__main__':  # pragma: no cover
